<?php
// $Id: content_type_loader.inc,v 1.1.2.4 2010/04/06 18:13:32 damienmckenna Exp $

/**
 * @file
 * Create/Read/Update/Delete functions for CCK-defined object types.
 */

/**
 * Make changes needed when a content type is created.
 *
 * @param $info
 *   value supplied by hook_node_type()
 *
 * node_get_types() is still missing the new type at this point due to
 * a static caching bug. We ask it to rebuild its cache so that
 * content_clear_type_cache() can do its job properly.
 */
function content_type_loader_create($info) {
  // Load all content type definitions.
  node_get_types(NULL, NULL, TRUE);

  // Load the CCK fields.
  content_type_loader_load_type($info);
  
  // Handle the variables.
  content_type_loader_set_module_vars($info);

  // Reset all content type info.
  content_clear_type_cache(TRUE);
  
  // Clear the menu cache.
  menu_cache_clear_all();
}

/**
 * Make changes needed when an existing content type is updated.
 *
 * @param $info
 *   value supplied by hook_node_type()
 */
function content_type_loader_update($info) {
  // Load the CCK fields.
  content_type_loader_load_type($info);

  // Handle the variables.
  content_type_loader_set_module_vars($info);

  // Reset all content type info.
  content_clear_type_cache(TRUE);

  // Clear the menu cache.
  menu_cache_clear_all();
}

/**
 * Make changes needed when a content type is deleted.
 *
 * @param $info
 *   value supplied by hook_node_type()
 *
 * TODO should we skip doing this entirely since core leaves the
 * nodes in the database as orphans and wait until the nodes are
 * deleted to respond?
 *
 */
function content_type_loader_delete($info) {
  // Don't delete data for content-types defined by disabled modules.
  if (!empty($info->disabled)) {
    return;
  }

  // Handle the variables.
  content_type_loader_del_module_vars($info);

  // Menu needs to be rebuilt as well, but node types need to be rebuilt first.
  // node_type_form_submit() takes care of this.
  content_clear_type_cache(TRUE);
}

/**
 * Load all of the field data for a given content type definition.
 */
function content_type_loader_load_type($info) {
  // Load the CCK CRUD functionality.
  module_load_include('inc', 'content', 'includes/content.crud');

  // Get the content type we are importing into.
  $type_name = $info->type;
  $type_label = node_get_types('name', $type_name);

  // Get all type and field info for this database.
  $content_info = _content_type_info();

  // It is allowed to import a type with no fields,
  // so the fields array could be empty and must be cast as an array.
  $imported_fields = isset($info->fields) ? $info->fields : array();

  // Perform more pre-import error trapping.
  // If there are potential problems, exit without doing the import.
  $not_enabled = array();

  // The groups array could be empty and still valid, make sure to cast it as an array.
  // If there are groups in the import, make sure the fieldgroup module is enabled.
  $imported_groups = array();
  if (isset($info->groups) && module_exists('fieldgroup')) {
    $imported_groups = (array) $info->groups;
  }
  elseif (isset($info->groups) && is_array($info->groups)) {
    $not_enabled[] = 'fieldgroup';
  }

  // Make sure that all the field and widget modules in the import are enabled in this database.
  foreach ($imported_fields as $import) {
    $field = content_field_instance_collapse($import);
    if (empty($field['module']) || empty($field['widget_module'])) {
      $not_enabled[] = $field['field_name'];
    }
    else {
      if (!module_exists($field['module'])) {
        $not_enabled[] = $field['module'];
      }
      if (!module_exists($field['widget_module'])) {
        $not_enabled[] = $field['widget_module'];
      }
    }
  }

  // If any required module is not enabled, set an error message and exit.
  if ($not_enabled) {
    form_set_error('macro', t('The following modules must be enabled for this content type to be loaded: %modules.', array(
        '%modules' => implode(', ', array_unique($not_enabled))
        )));
  }

  if (form_get_errors()) {
    drupal_set_message(t('Exiting. The "%type" content type was not fully installed, please correct the issues above and try again.', $info->type), 'error');
    return;
  }

  // Create the groups for this type, if they don't already exist.
  if (module_exists('fieldgroup') && $imported_groups) {
    foreach ($imported_groups as $group) {
      $group_name = $group['group_name'];
      fieldgroup_save_group($type_name, $group);
    }
    // Reset the static variable in fieldgroup_groups() with new data.
    fieldgroup_groups('', FALSE, TRUE);
  }

  // Iterate through the field forms in the import and execute each.
  $rebuild = FALSE;
  foreach ($imported_fields as $field) {

    // Make sure the field doesn't already exist in the type.
    // If so, do nothing, fields can't be duplicated within a content type.
    $field_name   = $field['field_name'];

    // Might need to overwrite the content type name if a new type was created.
    $field['type_name'] = $type_name;

    // TODO: support an optional way to refresh the settings.
    if (!empty($field['field_name']) && isset($content_info['content types'][$type_name]['fields'][$field_name])) {
      drupal_set_message(t('The imported field %field_label (%field_name) was not added to %type because that field already exists in %type.', array(
        '%field_label' => $field['label'], '%field_name' => $field_name, '%type' => $type_label)));
    }
    else {
      $field = content_field_instance_create($field, FALSE);
      $rebuild = TRUE;
      drupal_set_message(t('The field %field_label (%field_name) was added to the content type %type.', array(
        '%field_label' => $field['widget']['label'], '%field_name' => $field_name, '%type' => $type_label)));
    }

    // Fieldgroup module erases all group related data when a module that
    // provides a content type is disabled, but CCK does not remove the fields.
    // In this case, we should ensure group data related to fields is properly
    // restored. Hence, we need to update field group data for newly imported
    // field, but also for fields that already exist.
    if (module_exists('fieldgroup') && isset($imported_groups)) {
      fieldgroup_update_fields($field);
    }
  }
}

/**
 * Assign a set of variables for a module based on a defined list of known
 * settings.
 * @param $module
 *   The internal module name.
 * @param $settings
 *   The list of known variable names for this content type.
 * @param $info
 *   The content type's settings.
 */
function content_type_loader_set_module_vars($info) {
  // Obtain an array of all variables applicable for this content type.
  $variables = content_type_loader_get_all_variables($info->type);

  // Loop over the module settings array.
  foreach ($variables as $module => $settings) {
    // Loop over each of the module's settings.
    foreach ($settings as $x => $setting) {
      // Does the setting exist in the content type definition?
      if (!isset($info->$setting)) {
        // Is this module enabled?  Only do this check once.
        if ($x == 0 && !module_exists($module)) {
          drupal_set_message(t('Settings found for the @module module but it is not enabled.', array('@module' => $module)));
        }

        // See if the token value was added to the setting name.
        if (strpos($setting, CONTENT_HANDLE_VAR_TOKEN) !== FALSE) {
          $variable = str_replace(CONTENT_HANDLE_VAR_TOKEN, $info->type, $setting);
        }
        // The default is $setting_$type.
        else {
          $variable = $setting .'_'. $info->type;
        }

        // Assign the setting variable.
        variable_set($variable, $info->$variable);
      }
    }
  }
}

/**
 * Remove a set of variables for a module based on a defined list of known
 * settings.
 * @param $module
 *   The internal module name.
 * @param $settings
 *   The list of known variable names for this content type.
 * @param $info
 *   The content type's settings.
 */
function content_type_loader_del_module_vars($module, $info) {
  // Obtain an array of all variables applicable for this content type.
  $variables = content_type_loader_get_all_variables($info->type);

  // Loop over the module settings array.
  foreach ($variables as $module => $settings) {
    // Loop over each of the module's settings.
    foreach ($settings as $x => $setting) {
      // See if the token value was added to the setting name.
      if (strpos($setting, CONTENT_HANDLE_VAR_TOKEN) !== FALSE) {
        $variable = str_replace(CONTENT_HANDLE_VAR_TOKEN, $info->type, $setting);
      }
      // The default is $setting_$type.
      else {
        $variable = $setting .'_'. $info->type;
      }
      // Remove the setting variable.
      variable_del($variable);
    }
  }
}

/**
 * Obtain a nested array of all content type variables for a given type.
 *
 * @param $type
 *   The content type to build the variables list for.
 * @return
 *   The nested array of variables for this content type.
 */
function content_type_loader_get_all_variables($type) {
  // hook_content_type_variables($type)
  // Control addition and removal of variables related to content types using
  // via the hook_node_info() and hook_node_type() hooks. Elements are added
  // to the array $variables in the format: array('variable1', 'variable2');
  //
  // function mymodule_content_type_variables($type) {
  //   $variables = array(
  //     'variable1', // '_typename' will be added.
  //     'variable2', // '_typename' will be added.
  //     // CONTENT_HANDLE_VAR_TOKEN will be replaced with 'typename'.
  //     'var_'. CONTENT_HANDLE_VAR_TOKEN .'_able',
  //   );
  //   return $variables;
  // }

  $hook = 'content_type_variables';
  
  // Loop over each module implementation.
  foreach (module_implements($hook) as $module) {
    $function = $module .'_'. $hook;
    $result = call_user_func_array($function, $type);
    // If anything was found, add it to the array.
    if (!empty($result)) {
      $variables[$module] = $result;
    }
  }

  // hook_content_type_variables($type)
  // All the values to be altered.  Variable names are stored in the format:
  // $variables['modulename']['variablename'];
  drupal_alter('content_type_variables', $variables);
  
  return $variables;
}
